# -*- coding: utf-8 -*-

# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from google.cloud.aiplatform import base
from google.cloud.aiplatform.featurestore import _entity_type
from google.cloud.aiplatform.preview.featurestore import entity_type


class EntityType(_entity_type._EntityType, base.PreviewMixin):
    """Public managed EntityType resource for Vertex AI."""

    _preview_class = entity_type.EntityType
